#include <string.h>
#include <math.h>

#include <winsock2.h>
#include <ws2tcpip.h>
#pragma comment(lib, "ws2_32.lib")
#include <windows.h>

#include "GUI.h"

using namespace System;
using namespace System::Runtime::InteropServices;
using namespace Microsoft::Win32;

using namespace ExtIO_RedPitaya_122_88;

//---------------------------------------------------------------------------

#define EXTIO_API __declspec(dllexport) __stdcall

#define TCP_PORT 1001
#define TCP_ADDR "192.168.1.100"

SOCKET gSock[2] = {INVALID_SOCKET, INVALID_SOCKET};

char gBuffer[8192];

UInt32 gRate = 96000;
Int32 gCorr = 0;

UInt32 gFreq = 600000;
UInt32 gFreqMin = 48000;
UInt32 gFreqMax = 60000000;

bool gInitHW = false;

bool gExitThread = false;
bool gThreadRunning = false;

//---------------------------------------------------------------------------

ref class ManagedGlobals
{
  public:
    static GUI ^gGUI = nullptr;
    static RegistryKey ^gKey = nullptr;
};

//---------------------------------------------------------------------------

void (*ExtIOCallback)(int, int, float, void *) = 0;

static SOCKET OpenConnection(char *buffer);
static void SetRate(UInt32);
static void UpdateRate(UInt32);
static void SetCorr(Int32);

//---------------------------------------------------------------------------

DWORD WINAPI GeneratorThreadProc(__in LPVOID lpParameter)
{
  unsigned long size = 0;

  while(!gExitThread)
  {
    SleepEx(1, FALSE);

    if(gExitThread) break;

    ioctlsocket(gSock[1], FIONREAD, &size);

    while(size >= 8192)
    {
      recv(gSock[1], gBuffer, 8192, 0);

      if(ExtIOCallback) (*ExtIOCallback)(1024, 0, 0.0, gBuffer);

      ioctlsocket(gSock[1], FIONREAD, &size);
    }
  }
  gExitThread = false;
  gThreadRunning = false;
  return 0;
}

//---------------------------------------------------------------------------

static void StopThread()
{
  if(gThreadRunning)
  {
    gExitThread = true;
    while(gThreadRunning)
    {
      SleepEx(10, FALSE);
    }
  }
}

//---------------------------------------------------------------------------

static void StartThread()
{
  gExitThread = false;
  gThreadRunning = true;
  CreateThread(NULL, (SIZE_T)(64 * 1024), GeneratorThreadProc, NULL, 0, NULL);
}

//---------------------------------------------------------------------------

extern "C" int EXTIO_API SetHWLO(long LOfreq);

//---------------------------------------------------------------------------

extern "C"
bool EXTIO_API InitHW(char *name, char *model, int &type)
{
  String ^addrString;
  UInt32 rateIndex = 2;
  Int32 corr = 0;

  type = 7;

  strcpy(name, "Red Pitaya SDR 122.88");
  strcpy(model, "");

  if(!gInitHW)
  {
    ManagedGlobals::gKey = Registry::CurrentUser->OpenSubKey("Software\\ExtIO_RedPitaya_122_88", true);
    if(!ManagedGlobals::gKey)
    {
      ManagedGlobals::gKey = Registry::CurrentUser->CreateSubKey("Software\\ExtIO_RedPitaya_122_88");
      ManagedGlobals::gKey->SetValue("IP Address", TCP_ADDR);
      ManagedGlobals::gKey->SetValue("Sample Rate", rateIndex);
      ManagedGlobals::gKey->SetValue("Freq. Corr.", corr);
    }

    ManagedGlobals::gGUI = gcnew GUI;
    addrString = ManagedGlobals::gKey->GetValue("IP Address")->ToString();
    ManagedGlobals::gGUI->addrValue->Text = addrString;

    rateIndex = Convert::ToUInt32(ManagedGlobals::gKey->GetValue("Sample Rate", 1));
    if(rateIndex < 0 || rateIndex > 6) rateIndex = 2;
    ManagedGlobals::gGUI->rateValue->SelectedIndex = rateIndex;
    ManagedGlobals::gGUI->rateCallback = UpdateRate;

    corr = Convert::ToInt32(ManagedGlobals::gKey->GetValue("Freq. Corr.", 0));
    if(corr < -100 || corr > 100) corr = 0;
    ManagedGlobals::gGUI->corrValue->Value = corr;
    ManagedGlobals::gGUI->corrCallback = SetCorr;

    SetRate(rateIndex);
    SetCorr(corr);

    gInitHW = true;
  }

  return gInitHW;
}

//---------------------------------------------------------------------------

extern "C"
bool EXTIO_API OpenHW()
{
  return gInitHW;
}

//---------------------------------------------------------------------------

extern "C"
int EXTIO_API StartHW(long LOfreq)
{
  WSADATA wsaData;
  struct sockaddr_in addr;
  String ^addrString;
  char *buffer;
  UInt32 command;
  int i;

  if(!gInitHW) return 0;

  WSAStartup(MAKEWORD(2, 2), &wsaData);

  addrString = ManagedGlobals::gGUI->addrValue->Text;
  ManagedGlobals::gKey->SetValue("IP Address", addrString);

  buffer = (char *)Marshal::StringToHGlobalAnsi(addrString).ToPointer();

  gSock[0] = OpenConnection(buffer);
  gSock[1] = OpenConnection(buffer);

  Marshal::FreeHGlobal(IntPtr(buffer));

  if(gSock[0] == INVALID_SOCKET || gSock[1] == INVALID_SOCKET)
  {
    closesocket(gSock[1]);
    closesocket(gSock[0]);
    return 0;
  }

  command = 0;
  send(gSock[0], (char *)&command, 4, 0);
  command = 1;
  send(gSock[1], (char *)&command, 4, 0);

  StopThread();
  gFreq = LOfreq;
  SetRate(ManagedGlobals::gGUI->rateValue->SelectedIndex);
  StartThread();

  return 1024;
}

//---------------------------------------------------------------------------

extern "C"
void EXTIO_API StopHW()
{
  StopThread();

  closesocket(gSock[1]);
  closesocket(gSock[0]);

  gSock[1] = INVALID_SOCKET;
  gSock[0] = INVALID_SOCKET;

  WSACleanup();
}

//---------------------------------------------------------------------------

extern "C"
void EXTIO_API CloseHW()
{
  gInitHW = false;
}

//---------------------------------------------------------------------------

extern "C"
void EXTIO_API SetCallback(void (*callback)(int, int, float, void *))
{
  ExtIOCallback = callback;
}

//---------------------------------------------------------------------------

extern "C"
int EXTIO_API SetHWLO(long LOfreq)
{
  int rc = 0;
  UInt32 buffer = 0;

  gFreq = LOfreq;

  // check limits
  if(gFreq < gFreqMin)
  {
    gFreq = gFreqMin;
    rc = -gFreqMin;
  }
  else if(gFreq > gFreqMax)
  {
    gFreq = gFreqMax;
    rc = gFreqMax;
  }

  if(gFreq != LOfreq && ExtIOCallback) (*ExtIOCallback)(-1, 101, 0.0, 0);

  buffer = (long)floor(gFreq*(1.0 + gCorr*1.0e-6) + 0.5);
  if(gSock[0] > 0) send(gSock[0], (char *)&buffer, 4, 0);

  return rc;
}

//---------------------------------------------------------------------------

static SOCKET OpenConnection(char *buffer)
{
  struct sockaddr_in addr;
  fd_set writefds, exceptfds;
  struct timeval timeout;
  int result;
  SOCKET sock;
  u_long mode;

  sock = socket(AF_INET, SOCK_STREAM, 0);

  if(sock == INVALID_SOCKET)
  {
    return INVALID_SOCKET;
  }

  mode = 1;
  ioctlsocket(sock, FIONBIO, &mode);

  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  inet_pton(AF_INET, buffer, &addr.sin_addr);
  addr.sin_port = htons(TCP_PORT);

  connect(sock, (struct sockaddr *)&addr, sizeof(addr));

  timeout.tv_sec = 5;
  timeout.tv_usec = 0;

  FD_ZERO(&writefds);
  FD_SET(sock, &writefds);
  FD_ZERO(&exceptfds);
  FD_SET(sock, &exceptfds);

  result = select(0, NULL, &writefds, &exceptfds, &timeout);

  if(result == 0 || result == SOCKET_ERROR)
  {
    closesocket(sock);
    return INVALID_SOCKET;
  }

  mode = 0;
  ioctlsocket(sock, FIONBIO, &mode);

  return sock;
}

//---------------------------------------------------------------------------

static void SetRate(UInt32 rateIndex)
{
  switch(rateIndex)
  {
    case 0: gRate = 24000; gFreqMin = 12000; break;
    case 1: gRate = 48000; gFreqMin = 24000; break;
    case 2: gRate = 96000; gFreqMin = 48000; break;
    case 3: gRate = 192000; gFreqMin = 96000; break;
    case 4: gRate = 384000; gFreqMin = 192000; break;
    case 5: gRate = 768000; gFreqMin = 384000; break;
    case 6: gRate = 1536000; gFreqMin = 768000; break;
  }

  if(ManagedGlobals::gKey) ManagedGlobals::gKey->SetValue("Sample Rate", rateIndex);

  rateIndex |= 1<<28;
  if(gSock[0] > 0) send(gSock[0], (char *)&rateIndex, 4, 0);

  SetHWLO(gFreq);
}

//---------------------------------------------------------------------------

static void UpdateRate(UInt32 rateIndex)
{
  SetRate(rateIndex);
  if(ExtIOCallback) (*ExtIOCallback)(-1, 100, 0.0, 0);
}

//---------------------------------------------------------------------------

static void SetCorr(Int32 corr)
{
  gCorr = corr;
  if(ManagedGlobals::gKey) ManagedGlobals::gKey->SetValue("Freq. Corr.", corr);
  SetHWLO(gFreq);
}

//---------------------------------------------------------------------------

extern "C"
long EXTIO_API GetHWLO()
{
  return gFreq;
}

//---------------------------------------------------------------------------

extern "C"
long EXTIO_API GetHWSR()
{
  return gRate;
}

//---------------------------------------------------------------------------

extern "C"
int EXTIO_API GetStatus()
{
  return 0;
}

//---------------------------------------------------------------------------

extern "C"
void EXTIO_API ShowGUI()
{
  if(ManagedGlobals::gGUI) ManagedGlobals::gGUI->ShowDialog();
}
